// Generic-FPA-TS.h : main header file for the Generic-FPA-TS DLL
//

#ifndef GENERIC_FPA_TS_H
#define GENERIC_FPA_TS_H

//top-DLL : this DLL. It is a thread-safe wrapper for calling adapter DLLs.  
//sub-DLL : MSP430FPA.dll, FlashProARM.dll, etc.
typedef struct
{
	int fpa_index;
	int debug_mode;//set to 1 if testing for sub-DLL functions implemented.  If set to 1 sub-DLL functions won't get called.	
	int dll_call_status;
} thread_struct_t;

#ifndef GENERIC_FPA_H
//dll_call_status is set after each invocation of this top-DLL.  Sub-DLL return values are passed using normal return statement.
//ELP_CALLED - specific DLL function is implemented and was called, but says nothing about function working/not working
//ELP_BUSY - DLL is busy serving another thread
//ELP_IMPL - specific DLL function is implemented for this DLL type (returned instead of ELP_CALLED if debug_mode == 1)
//ELP_ENOTIMP - specific DLL function is not implemented for this DLL type
//ELP_ERANGE - thread info block wants DLL index out of range
//ELP_ENOINIT - DLL specified by thread info block did not initialize, or failed at initialization
#define	ELP_CALLED	-8
#define	ELP_BUSY	-9
#define	ELP_IMPL	-10
#define	ELP_ENOTIMP	-11
#define	ELP_ERANGE	-12
#define	ELP_ENOINIT	-13

typedef unsigned char BYTE;
typedef int INT_X;

#ifdef __GNUC__
typedef unsigned long long ULONG64;
#else
typedef unsigned _int64   ULONG64;
#endif

#define		CONFSEL_VALIDATE	0 
#define		CONFSEL_VALUE		1 
#define		CONFSEL_MIN			2 
#define		CONFSEL_MAX			3 
#define		CONFSEL_DEFAULT		4 

#define		REPORT_MESSAGE_MAX_SIZE		2000
#define		MAX_USB_DEV_NUMBER			64
#define		MAX_FPA_INDEX				64
#define		ALL_ACTIVE_FPA				0
#define		FPA_UNMATCHED_RESULTS		-1
#define		ANY_FPA						-4

#define		PRG_ADAPTER_NOT_DETECTED	4	
#define		TERMINATED_VCC_TOO_LOW		0x20

#define		CODE_IN_ROM					0x10	
#define		CODE_IN_RAM					0x20
#define		CODE_OUT_OF_FLASH			0x40
#define		CODE_OVERWRITTEN			0x80
#define		PASSWORD_NOT_FOUND			0x100
#define		INVALID_CODE_FILE			0x200
#define		OPEN_FILE_OR_READ_ERR		0x8

#define		C2000_DLL_TYPE			"C2000"
#define		CC_DLL_TYPE				"CC"
#define		CC_GP_DLL_TYPE			"CC-GP"
#define		ARM_DLL_TYPE			"ARM"
#define		ARM_GP_DLL_TYPE			"ARM-GP"
#define		MSP_DLL_TYPE			"MSP"
#define		MSP_GP_DLL_TYPE			"MSP-GP"
#define		FPM_DLL_TYPE			"M"
#define		GPM_DLL_TYPE			"M-GP"
#define		IMOTION_DLL_TYPE		"IMOTION"
#define		IMOTION_GP_DLL_TYPE		"IMOTION-GP"

#define		GANG_SIZE	6

extern "C"
{
	//Should be the first function called if local DLL path needs to be changed
	//Path to directory where API-DLLs are present: MSP430FPA1.dll, FlashPro2000-FPA1.dll, FlashProARM-FPA1.dll, FlashProCC-FPA1.dll, GangPro430FPA1.dll, GangProARM-FPA1.dll, GangProCC-FPA1.dll, FlashProiMOTION-FPA1.dll, GangProiMOTION-FPA1.dll
	//Max Path size 512 bytes
	//Returns TRUE if copied positive number of bytes less than max size
	//Returns FALSE otherwise	
	INT_X	__stdcall	TS_APIDLL_Directory(char * FilePath);

	//pass configuration file.  The file is used to open the desired DLL types
	//returns number of device DLLs initialized
	INT_X	__stdcall	TS_AssignFPAs(char *FileName);

	//pass type of DLL to use for search, C2000_DLL_TYPE, CC_DLL_TYPE, ARM_DLL_TYPE, MSP_DLL_TYPE, etc. Make sure selected API-DLL is present.
	//Any API-DLL will search for all types of FPAs connected.
	//return all FPAs connected to computer, with serial number and type etc., up to 64
	//returned text is same as F_Get_FPA_Label for each FPA.
	char *	__stdcall	TS_DiscoverFPAs( char *type );
  
	//pass fpa index and fpa type  
	//return TRUE(1) if successful, FALSE(0) is unsuccessful.
	//will fail if fpa index already taken, and/or type is invalid
	INT_X	__stdcall	TS_OpenOneInstance( thread_struct_t *ti );
  
	//close device at index fpa
	//return TRUE(1) if successful, FALSE(0) is unsuccessful or instance wasn't open to begin with.
	INT_X	__stdcall	TS_CloseOneInstance( thread_struct_t *ti );
  
	//explicitly unloads all previously loaded device DLLs and frees associated data structures
	INT_X	__stdcall	TS_CloseInstances( void );

	INT_X	__stdcall	TS_Check_FPA_access(thread_struct_t * ti, INT_X index);	
	INT_X	__stdcall	TS_Check_FPA_access_specDLL(thread_struct_t *ti, INT_X index, UINT32 extra);
	INT_X	__stdcall	TS_GenericTSDLLVer(thread_struct_t *ti);//  return	Version = 0x0FFF & version
	INT_X	__stdcall	TS_Get_FPA_SN(thread_struct_t * ti, BYTE fpa);
	
	INT_X	__stdcall	TS_Get_FPA_Label(thread_struct_t * ti, BYTE fpa, BYTE * label);
	INT_X	__stdcall	TS_Get_Power_Results(thread_struct_t *ti, double *xs_adc);
	INT_X	__stdcall	TS_Get_Last_Power_Results_data(thread_struct_t *ti, double *xs_adc);
	INT_X   __stdcall   TS_XS_Current_zeroing(thread_struct_t *ti, int state, char *txt);
	INT_X	__stdcall	TS_XS_Update_HW_State(thread_struct_t *ti, BYTE mode, INT_X data);
	INT_X	__stdcall	TS_XS_Clr_ADC_av_history(thread_struct_t *ti);

	INT_X	__stdcall	TS_GetProgressBar(thread_struct_t *ti, BYTE fpa);//not covered by mutex; can be invoked while different function on same FPA is in progress (AutoProgram, etc.)
	INT_X	__stdcall	TS_GetLastOpCode(thread_struct_t *ti, BYTE fpa);//not covered by mutex; can be invoked while different function on same FPA is in progress (AutoProgram, etc.)
	INT_X	__stdcall	TS_CancelAction(thread_struct_t *ti, BYTE fpa);//not covered by mutex; can be invoked while different function on same FPA is in progress (AutoProgram, etc.)

	//returns a char * representing the type of selected FPA
	// NULL if FPA out of range or selected FPA not initialized
	const char *	__stdcall	TS_Get_FPA_type(thread_struct_t *ti);
	
	//====================================================================Generic Functions
	INT_X	__stdcall	TS_Initialization(thread_struct_t * ti);
	INT_X	__stdcall	TS_Use_Config_INI(thread_struct_t *ti, BYTE use);
	char *	__stdcall	TS_Get_Config_Name_List(thread_struct_t *ti, INT_X index);
	unsigned int	__stdcall	TS_Get_Config_Value_By_Name(thread_struct_t *ti, char *name, int type);
	INT_X	__stdcall	TS_Set_Config_Value_By_Name(thread_struct_t *ti, char *name, unsigned int newValue);

	INT_X	__stdcall	TS_SetConfig(thread_struct_t * ti, INT_X index, INT_X data);
	INT_X	__stdcall	TS_GetConfig(thread_struct_t * ti, INT_X index);
	void	__stdcall   TS_Reset_Config(thread_struct_t *ti);

	INT_X	__stdcall	TS_Get_Device_Info(thread_struct_t * ti, INT_X index);
	INT_X	__stdcall	TS_Set_MCU_Name(thread_struct_t * ti, char * MCU_name);
	void*   __stdcall	TS_Search_MCU_Name(thread_struct_t *ti, char * MCU_name_regex, char * Vendor_filter, int mode, int flags);
	char*	__stdcall	TS_ARM_Get_MCU_Name_list(thread_struct_t *ti, INT_X type, INT_X index);
	INT_X	__stdcall	TS_ARM_Set_MCU_Family_Group(thread_struct_t *ti, INT_X type, INT_X index);
	void	__stdcall	TS_ReportMessage(thread_struct_t * ti, char * text);
	char *	__stdcall	TS_Report_Message(thread_struct_t * ti);
	BYTE	__stdcall	TS_GetReportMessageChar(thread_struct_t * ti, INT_X index);
	INT_X	__stdcall	TS_DLLTypeVer(thread_struct_t * ti);
	INT_X	__stdcall	TS_ConfigFileLoad(thread_struct_t * ti, char * filename);
	INT_X	__stdcall   TS_LoadFromImage(thread_struct_t *ti, char * fullPath, char *password);
	INT_X	__stdcall	TS_Power_Target(thread_struct_t * ti, INT_X OnOff);
	INT_X	__stdcall	TS_Reset_Target(thread_struct_t * ti);
	INT_X	__stdcall	TS_Get_Targets_Vcc(thread_struct_t * ti);
	INT_X	__stdcall	TS_Set_fpa_io_state(thread_struct_t * ti, BYTE debug, BYTE reset, BYTE VccOn);
	INT_X	__stdcall	TS_Set_fpa_IO(thread_struct_t * ti, BYTE jtag, BYTE jtag_reset, BYTE EMU0, BYTE EMU1);
	INT_X	__stdcall	TS_Get_Sector_Size(thread_struct_t * ti, INT_X address);
	INT_X	__stdcall	TS_Get_Lock_Bits(thread_struct_t * ti, BYTE target_no);
	
	//====================================================================Data Buffer Functions
	INT_X	__stdcall	TS_ReadCodeFile(thread_struct_t * ti, char * FileName);
	INT_X	__stdcall	TS_MSP_ReadCodeFile(thread_struct_t * ti, INT_X file_format, char * FileName);
	INT_X	__stdcall	TS_CC_ReadCodeFile(thread_struct_t * ti, INT_X file_format, char * FileName);
	INT_X	__stdcall	TS_AppendCodeFile(thread_struct_t * ti, char * FileName);
	INT_X	__stdcall	TS_Get_CodeCS(thread_struct_t * ti, INT_X dest);
	INT_X	__stdcall	TS_ReadPasswFile(thread_struct_t * ti, char * FileName);
	INT_X	__stdcall	TS_MSP_ReadPasswFile(thread_struct_t * ti, INT_X file_format, char * FileName);
	INT_X	__stdcall	TS_Clr_Code_Buffer(thread_struct_t * ti);
	
	INT_X	__stdcall	TS_Put_Byte_to_Password_Buffer(thread_struct_t * ti, INT_X addr, BYTE data);
	INT_X	__stdcall	TS_Get_Byte_from_Password_Buffer(thread_struct_t * ti, INT_X addr);

	INT_X	__stdcall	TS_Put_Byte_to_Buffer(thread_struct_t * ti, INT_X addr, BYTE data);
	BYTE	__stdcall	TS_Get_Byte_from_Buffer(thread_struct_t * ti, INT_X addr);
	INT_X	__stdcall	TS_Put_Word_to_Buffer(thread_struct_t * ti, INT_X addr, INT_X data);
	INT_X	__stdcall	TS_Get_Word_from_Buffer(thread_struct_t * ti, INT_X addr);
	void	__stdcall	TS_Put_IEEEAddr64_to_Buffer(thread_struct_t * ti, ULONG64 IEEEAddr);
	ULONG64	__stdcall	TS_Get_IEEEAddr64_from_Buffer(thread_struct_t * ti);
	INT_X	__stdcall	TS_Put_IEEEAddr_Byte_to_Buffer(thread_struct_t * ti, BYTE no, BYTE IEEEAddr);
	BYTE	__stdcall	TS_Get_IEEEAddr_Byte_from_Buffer(thread_struct_t * ti, BYTE no);
	INT_X	__stdcall	TS_Copy_Buffer_to_direct_RAM(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_direct_RAM_to_Buffer(thread_struct_t * ti, INT_X start_addr, INT_X size);
		
	INT_X	__stdcall	TS_Put_Byte_to_Code_Buffer(thread_struct_t * ti, INT_X addr, BYTE data);
	INT_X	__stdcall	TS_Get_Byte_from_Code_Buffer(thread_struct_t * ti, INT_X addr);
	INT_X	__stdcall	TS_Put_Word_to_Code_Buffer(thread_struct_t * ti, INT_X addr, INT_X data);
	INT_X	__stdcall	TS_Get_Word_from_Code_Buffer(thread_struct_t * ti, INT_X addr);

	INT_X	__stdcall	TS_Put_Word_to_CSM_Buffer(thread_struct_t * ti, INT_X dest, INT_X addr, INT_X data);
	INT_X	__stdcall	TS_Get_Word_from_CSM_Buffer(thread_struct_t * ti, INT_X dest, INT_X addr);
	
	INT_X	__stdcall	TS_Put_Byte_to_Gang_Buffer(thread_struct_t * ti, BYTE target_no, INT_X addr, BYTE data);
	BYTE 	__stdcall	TS_Get_Byte_from_Gang_Buffer(thread_struct_t * ti, BYTE target_no, INT_X addr);
	void	__stdcall 	TS_Put_IEEEAddr64_to_Gang_Buffer(thread_struct_t * ti, BYTE target_no, ULONG64 IEEEAddr);
	ULONG64	__stdcall	TS_Get_IEEEAddr64_from_Gang_Buffer(thread_struct_t * ti, BYTE target_no);
	INT_X	__stdcall 	TS_Put_IEEEAddr_Byte_to_Gang_Buffer(thread_struct_t * ti, BYTE target_no, BYTE no, BYTE IEEEAddr);
	BYTE	__stdcall	TS_Get_IEEEAddr_Byte_from_Gang_Buffer(thread_struct_t * ti, BYTE target_no, BYTE no);

	//====================================================================Encapsulated Functions
	INT_X	__stdcall	TS_AutoProgram(thread_struct_t * ti, INT_X mode);
	INT_X   __stdcall   TS_Get_Error_No(thread_struct_t * ti, BYTE fpa);
	INT_X	__stdcall	TS_Verify_Lock_Bits(thread_struct_t *ti);
	INT_X	__stdcall	TS_Verify_Access_to_MCU(thread_struct_t * ti);
	INT_X	__stdcall	TS_Verify_CSM_Password(thread_struct_t * ti);
	INT_X	__stdcall	TS_VerifyFuseOrPassword(thread_struct_t * ti);
	INT_X	__stdcall	TS_Memory_Erase(thread_struct_t * ti, INT_X mode);
	INT_X	__stdcall	TS_Memory_Blank_Check(thread_struct_t * ti);
	INT_X	__stdcall	TS_Memory_Write(thread_struct_t * ti, INT_X mode);
	INT_X	__stdcall	TS_Memory_Verify(thread_struct_t * ti, INT_X mode);	
	INT_X	__stdcall	TS_MSP_Memory_Read(thread_struct_t * ti, unsigned char * data);
	INT_X	__stdcall	TS_Memory_Read(thread_struct_t * ti);
	INT_X	__stdcall	TS_Gang_Flash_Read(thread_struct_t * ti);
	INT_X	__stdcall	TS_Write_IEEE_Address(thread_struct_t * ti);
	INT_X	__stdcall	TS_Read_IEEE_Address(thread_struct_t * ti);
	INT_X	__stdcall	TS_Copy_All_Flash_to_Buffer(thread_struct_t * ti);
	INT_X	__stdcall	TS_Restore_JTAG_Security_Fuse(thread_struct_t * ti);
	INT_X	__stdcall 	TS_Clear_Locked_Device(thread_struct_t *ti);
	INT_X	__stdcall	TS_Lock_MCU(thread_struct_t *ti);
	INT_X	__stdcall	TS_Write_CSM_Password(thread_struct_t * ti);	
	INT_X	__stdcall	TS_Write_Lock_Bits(thread_struct_t * ti);

	//====================================================================Sequential Functions
	INT_X	__stdcall	TS_Open_Target_Device(thread_struct_t * ti);
	INT_X	__stdcall	TS_Close_Target_Device(thread_struct_t * ti);
	INT_X	__stdcall	TS_Segment_Erase(thread_struct_t * ti, INT_X address); 
	INT_X	__stdcall	TS_Sectors_Blank_Check(thread_struct_t * ti, INT_X start_addr, INT_X stop_addr);
	INT_X	__stdcall	TS_Copy_Buffer_to_Flash(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_Flash_to_Buffer(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Set_PC_and_RUN(thread_struct_t * ti, INT_X PC_addr);
	INT_X	__stdcall	TS_CC_Set_PC_and_RUN(thread_struct_t * ti, INT_X xram_en, INT_X PC_addr);
	INT_X	__stdcall	TS_ARM_Write_Byte_to_RAM(thread_struct_t *ti, INT_X addr, BYTE data);
	INT_X	__stdcall	TS_ARM_Write_Word16_to_RAM(thread_struct_t *ti, INT_X addr, INT_X data);
	INT_X	__stdcall	TS_ARM_Write_Word32_to_RAM(thread_struct_t *ti, INT_X addr, INT_X data);
	INT_X	__stdcall	TS_ARM_Write_Bytes_Block_to_RAM(thread_struct_t *ti, INT_X start_addr, INT_X size, BYTE *data);
	INT_X	__stdcall	TS_ARM_Read_Byte(thread_struct_t *ti, INT_X addr);
	INT_X	__stdcall	TS_ARM_Read_Word16(thread_struct_t *ti, INT_X addr);
	INT_X	__stdcall	TS_ARM_Read_Word32(thread_struct_t *ti, INT_X addr);
	INT_X	__stdcall	TS_ARM_Read_Bytes_Block(thread_struct_t *ti, INT_X start_addr, INT_X size, BYTE *data);
	INT_X	__stdcall	TS_Write_Debug_Register(thread_struct_t *ti);
	INT_X	__stdcall	TS_Write_Locking_Registers(thread_struct_t *ti);
	INT_X	__stdcall	TS_Blow_Fuse(thread_struct_t * ti);
	INT_X	__stdcall	TS_Adj_DCO_Frequency(thread_struct_t * ti, INT_X freq_Hz);
	INT_X	__stdcall	TS_Test_DCO_Frequency(thread_struct_t * ti, INT_X DCO_constant);
	INT_X	__stdcall	TS_Get_DCO_constant(thread_struct_t * ti, BYTE target_no);
	INT_X	__stdcall	TS_Set_DCO_constant(thread_struct_t * ti, BYTE target_no);
	INT_X	__stdcall	TS_GP_Test_DCO_Frequency(thread_struct_t * ti);
	INT_X	__stdcall	TS_Get_DCO_Freq_result(thread_struct_t * ti, BYTE target_no);

	INT_X	__stdcall	TS_Write_Byte_to_XRAM(thread_struct_t * ti, INT_X addr, BYTE data);
	INT_X	__stdcall	TS_Read_Byte_from_XRAM(thread_struct_t * ti, INT_X addr);
	INT_X	__stdcall	TS_Write_Byte_to_direct_RAM(thread_struct_t * ti, INT_X addr, BYTE data);
	INT_X	__stdcall	TS_Read_Byte_from_direct_RAM(thread_struct_t * ti, INT_X addr);
	INT_X	__stdcall	TS_Copy_Buffer_to_XRAM(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_XRAM_to_Buffer(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Write_Word(thread_struct_t * ti, INT_X addr, INT_X data);
	INT_X	__stdcall	TS_Read_Word(thread_struct_t * ti, INT_X addr);
	INT_X	__stdcall	TS_Write_Byte(thread_struct_t * ti, INT_X addr, BYTE data);
	BYTE 	__stdcall	TS_Read_Byte(thread_struct_t * ti, INT_X addr);
	INT_X	__stdcall	TS_Memory_Write_Data(thread_struct_t * ti, INT_X start_addr, INT_X size, unsigned char * data);
	INT_X	__stdcall	TS_Memory_Read_Data(thread_struct_t * ti, INT_X start_addr, INT_X size, unsigned char * data);
	INT_X	__stdcall	TS_Copy_Buffer_to_RAM(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_RAM_to_Buffer(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Write_Word_to_RAM(thread_struct_t * ti, INT_X addr, INT_X data);

	INT_X	__stdcall	TS_Get_MCU_Data(thread_struct_t * ti, INT_X Type);
	INT_X	__stdcall	TS_Capture_PC_Addr(thread_struct_t * ti);
	INT_X	__stdcall	TS_GP_Capture_PC_Addr(thread_struct_t * ti, BYTE target_no);
	INT_X	__stdcall	TS_Synch_CPU_JTAG(thread_struct_t * ti);	

	INT_X   __stdcall	TS_Open_Comm(thread_struct_t *ti, INT_X activeReset);
	INT_X   __stdcall	TS_Soft_Reset_Target(thread_struct_t *ti);
	INT_X	__stdcall	TS_SetBoot0(thread_struct_t *ti, INT_X BOOT0_override, INT_X BOOT0_value);

	INT_X	__stdcall	TS_Copy_Gang_Buffer_to_XRAM(thread_struct_t * ti, INT_X  start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_XRAM_to_Gang_Buffer(thread_struct_t * ti, INT_X  start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_Gang_Buffer_to_direct_RAM(thread_struct_t * ti, INT_X  start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_direct_RAM_to_Gang_Buffer(thread_struct_t * ti, INT_X  start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_Gang_Buffer_to_Flash(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_Flash_to_Gang_Buffer(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_Gang_Buffer_to_RAM(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Copy_RAM_to_Gang_Buffer(thread_struct_t * ti, INT_X start_addr, INT_X size);
	INT_X	__stdcall	TS_Gang_Read_Byte(thread_struct_t * ti, BYTE target_no, INT_X addr);//any address, not only flash
	INT_X	__stdcall	TS_Gang_Read_Word16(thread_struct_t * ti, BYTE target_no, INT_X addr);//any address, not only flash
	INT_X	__stdcall	TS_Gang_Read_Word32(thread_struct_t * ti, BYTE target_no, INT_X addr);//any address, not only flash
	INT_X	__stdcall	TS_Gang_Read_Bytes_Block(thread_struct_t * ti, INT_X addr, INT_X size, BYTE data[][GANG_SIZE]);//any address, not only flash
	INT_X	__stdcall	TS_Copy_MCU_Data_to_Buffer(thread_struct_t * ti, INT_X Type);
	INT_X	__stdcall	TS_Get_MCU_Data_from_Buffer(thread_struct_t * ti, BYTE target_no);

	INT_X	__stdcall	TS_Get_Targets_Result(thread_struct_t * ti);
	INT_X	__stdcall	TS_Get_Active_Targets_Mask(thread_struct_t * ti);

	//====================================================================Customized JTAG instruction
	void 	__stdcall	TS_init_custom_jtag(thread_struct_t * ti, INT_X state, INT_X Vcc_index);
	INT_X	__stdcall	TS_custom_jtag_stream(thread_struct_t * ti, INT_X mode, INT_X size, BYTE * data_out, BYTE * data_in);

	//====================================================================UART
	INT_X	__stdcall	TS_Customize(thread_struct_t * ti, INT_X dest, INT_X data);
	INT_X	__stdcall	TS_Custom_Function(thread_struct_t * ti, INT_X type);	
}

#endif
#endif

